// This file is part of BOINC.
// https://boinc.berkeley.edu
// Copyright (C) 2026 University of California
//
// BOINC is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// BOINC is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
//

// pch.h : include file for standard system include files,
// or project specific include files that are used frequently, but
// are changed infrequently
//

#pragma once

#include <array>
#include <filesystem>
#include <fstream>
#include <ios>
#include <string>
#include <type_traits>
#include <vector>

#include <Windows.h>
#include <TlHelp32.h>
#include <Msi.h>
#include <MsiQuery.h>
#include <AclAPI.h>

#include "gtest/gtest.h"
#include "tinyxml2.h"
#include "wil/resource.h"
