/*
 * This file is part of BOINC.
 * https://boinc.berkeley.edu
 * Copyright (C) 2025 University of California
 *
 * BOINC is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 *
 * BOINC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.berkeley.boinc.rpc

const val ACCOUNT_MANAGER = "account_manager"
const val DESCRIPTION = "description"
const val ERROR_NUM = "error_num"
const val GUI_URL = "gui_url"
const val MASTER_URL = "master_url"
const val MESSAGE = "message"
const val NAME = "name"
const val NON_CPU_INTENSIVE = "non_cpu_intensive"
const val PLAN_CLASS = "plan_class"
const val PROJECT = "project"
const val PROJECT_NAME = "project_name"
const val PROJECT_URL = "project_url"
const val SEQNO = "seqno"
const val URL = "url"
const val USER_FRIENDLY_NAME = "user_friendly_name"
